@@ -30,6 +30,15 @@
 #' the BRNN method
 #' @param gam_k numeric, the parameter k for General Additive Model (GAM)
 #' @param gam_sp numeric, the parameter sp for General Additive Model (GAM)
+#' @param gom_a_range a numerical vector of the possible values of the
+#' parameter a, which is considered in the search for the initial Gompertz
+#' parameter values
+#' @param gom_b_range a numerical vector of the possible values of the
+#' parameter b, which is considered in the search for the initial Gompertz
+#' parameter values
+#' @param gom_k_range a numerical vector of the possible values of the
+#' parameter k, which is considered in the search for the initial Gompertz
+#' parameter values
 #'
 #' @return a list with the following elements:
 #' \enumerate{
@@ -49,9 +58,9 @@
 #' simulation_1 <- XPSgrowth(data_trees = data_trees,
 #'      parameters = parameters,
 #'      ID_vars = c("Species", "Tissue", "Site", "Year", "Tree"),
-#'      fitting_method = c("gompertz", "gam", "brnn"),
+#'      fitting_method = c("gam", "brnn"),
 #'      fitted_save = FALSE,
-#'      search_initial_gom = TRUE,
+#'      search_initial_gom = FALSE,
 #'      add_zeros = TRUE,
 #'      add_zeros_before = 'min',
 #'      post_process = TRUE)
@@ -67,7 +76,10 @@ XPSgrowth <- function(data_trees, parameters = NULL,
                  unified_parameters = FALSE,
                  gom_a = NA, gom_b = NA, gom_k = NA,
                  brnn_neurons = NA,
-                 gam_k = NA, gam_sp = NA){
+                 gam_k = NA, gam_sp = NA,
+                 gom_a_range = c(3000), gom_b_range = seq(50, 1000, 50),
+                 gom_k_range = seq(1, 500, 2)
+                 ){
 
   # Defining global variables
   Width <- NULL
@@ -79,6 +91,8 @@ XPSgrowth <- function(data_trees, parameters = NULL,
   errors_grid <- NA
   txtProgressBar <- NULL
   setTxtProgressBar <- NULL
+  lines <- NULL
+  note <- NULL
 
   # Progress bar
   pb <- txtProgressBar(min = 0, max = length(fitting_method), style = 3)
@@ -151,13 +165,20 @@ XPSgrowth <- function(data_trees, parameters = NULL,
 
   }
 
-
-
   # Create data key
-  data_trees$key <- apply(data_trees[, ID_vars], 1,
-                          paste0, sep = "", collapse = "_")
 
-  data_trees$key <- gsub(" ", "", data_trees$key) # remove empty characters
+  if (length(ID_vars) > 1){
+
+    data_trees$key <- apply(data_trees[, ID_vars], 1,
+                            paste0, sep = "", collapse = "_")
+
+    data_trees$key <- gsub(" ", "", data_trees$key) # remove empty characters
+
+  } else {
+
+    data_trees$key <- data_trees[, ID_vars]
+
+  }
 
   if (unified_parameters == TRUE){
 
@@ -172,10 +193,18 @@ XPSgrowth <- function(data_trees, parameters = NULL,
 
   } else {
 
-    parameters$key <- apply(parameters[, ID_vars], 1,
-                            paste0, sep = "", collapse = "_")
+    if (length(ID_vars) > 1){
 
-    parameters$key <- gsub(" ", "", parameters$key) # remove empty characters
+      parameters$key <- apply(parameters[, ID_vars], 1,
+                              paste0, sep = "", collapse = "_")
+
+      parameters$key <- gsub(" ", "", parameters$key) # remove empty characters
+
+    } else {
+
+      parameters$key <- parameters[, ID_vars]
+
+    }
 
   }
 
@@ -198,7 +227,6 @@ XPSgrowth <- function(data_trees, parameters = NULL,
 
   unique_keys <- unique(data_trees$key)
 
-
 for (ut in fitting_method){
 
   setTxtProgressBar(pb, pbar_holder)
@@ -209,10 +237,6 @@ if (current_fitting_method == "gompertz"){
 
   for (i in unique_keys){
 
-    gom_a_range <- 3000
-    gom_b_range <- seq(50, 1000, 50)
-    gom_k_range <- seq(1, 500, 2)
-
     par_grid <- expand.grid(gom_a = gom_a_range,
                             gom_b = gom_b_range,
                             gom_k = gom_k_range)
@@ -225,6 +249,9 @@ if (current_fitting_method == "gompertz"){
 
     temp_data <- data_trees[data_trees$key == i,]
 
+    # specify the measurement type - so we can distinguish from added zeros
+    temp_data$note <- "raw measurement"
+
     # add zeros at the beginning
     if(add_zeros == TRUE){
 
@@ -253,6 +280,7 @@ if (current_fitting_method == "gompertz"){
       new_rows <- do.call(rbind, row_list)
       new_rows$DOY <- c(1:min_DOY)
       new_rows$Width <- 0
+      new_rows$note <- "added zero"
       temp_data <- rbind(new_rows, temp_data)
     }
 
@@ -262,10 +290,15 @@ if (current_fitting_method == "gompertz"){
                                   minFactor = 1/1024, printEval = FALSE,
                                   warnOnly = FALSE)), silent = TRUE))
 
-    if (class(output) == "nls"){
+    # When all parameters are null, the class can still be nls. Additional check is needed
+    parm_test <- c(gom_a, gom_b, gom_k)
+
+    if (class(output) == "nls" & !is.null(parm_test)){
 
       temp_data$Width_pred <- predict(output)
 
+      temp_data <- dplyr::arrange(temp_data, DOY)
+
       # all what is below 0.1 goes to 0
       if (post_process == TRUE){
       temp_data$Width_pred <- ifelse(temp_data$Width_pred  < 0.01, 0,
@@ -278,10 +311,12 @@ if (current_fitting_method == "gompertz"){
       b_holder = b_holder + 1
 
       if (fitted_save == TRUE){
+
         ggplot(temp_data, aes(x = DOY, y = Width_pred)) + geom_line() +
-          geom_point(temp_data, mapping = aes(x = DOY, y = Width)) +
-          ylab("Width predicted") + theme_light()
-        ggsave(paste0("gom_", i, ".png"))
+          geom_point(temp_data, mapping = aes(x = DOY, y = Width, alpha = note)) +
+          ylab("Width predicted") + theme_light() + guides(alpha = "none")
+
+        ggsave(paste0("gom_", i, ".png"), width = 7, height = 6)
       }
 
     } else {
@@ -309,6 +344,8 @@ if (current_fitting_method == "gompertz"){
 
           temp_data$Width_pred <- predict(output)
 
+          temp_data <- dplyr::arrange(temp_data, DOY)
+
           temp_data$method <- "gompertz"
           temp_data$first_diff <- NULL
 
@@ -316,10 +353,12 @@ if (current_fitting_method == "gompertz"){
           b_holder = b_holder + 1
 
           if (fitted_save == TRUE){
+
             ggplot(temp_data, aes(x = DOY, y = Width_pred)) + geom_line() +
-              geom_point(temp_data, mapping = aes(x = DOY, y = Width)) +
-              ylab("Width predicted") + theme_light()
-            ggsave(paste0("gom_", i, ".png"))
+              geom_point(temp_data, mapping = aes(x = DOY, y = Width, alpha = note)) +
+              ylab("Width predicted") + theme_light() + guides(alpha = "none")
+
+            ggsave(paste0("gom_", i, ".png"), width = 7, height = 6)
           }
 
           list_solution_a[[p2]] <- gom_a
@@ -357,6 +396,9 @@ if (current_fitting_method == "gompertz"){
     temp_data <- data_trees[data_trees$key == i,]
     temp_neurons <- data_neurons[data_neurons$key == i, ]
 
+    # specify the measurement type - so we can distinguish from added zeros
+    temp_data$note <- "raw measurement"
+
       # add zeros at the beginning
     if (add_zeros == TRUE){
 
@@ -385,17 +427,23 @@ if (current_fitting_method == "gompertz"){
       new_rows <- do.call(rbind, row_list)
       new_rows$DOY <- c(1:min_DOY)
       new_rows$Width <- 0
+      new_rows$note <- "added zero"
       temp_data <- rbind(new_rows, temp_data)
 
     }
 
       capture.output(output <- try(brnn(Width ~ DOY, data = temp_data,
-                     neurons = temp_neurons$brnn_neurons), silent=TRUE))
+                     neurons = temp_neurons$brnn_neurons[1]), silent=TRUE))
 
       temp_data$Width_pred <- predict(output)
 
+      temp_data <- dplyr::arrange(temp_data, DOY)
+
       # Prediction for DOY 1 must always be 0
-      temp_data[1, "Width_pred"] <- 0
+      # temp_data[1, "Width_pred"] <- 0
+
+      # plot(y = temp_data$Width, x = temp_data$DOY, main = i)
+      # lines(y = temp_data$Width_pred, x = temp_data$DOY, type = "l")
 
       if (post_process == TRUE){
 
@@ -456,6 +504,9 @@ if (current_fitting_method == "gompertz"){
       }
     }
 
+      # plot(y = temp_data$Width, x = temp_data$DOY, main = i)
+      # lines(y = temp_data$Width_pred, x = temp_data$DOY, type = "l")
+
       temp_data$first_diff <- NULL
       temp_data$method <- "brnn"
 
@@ -463,10 +514,12 @@ if (current_fitting_method == "gompertz"){
       b_holder = b_holder + 1
 
       if (fitted_save == TRUE){
+
         ggplot(temp_data, aes(x = DOY, y = Width_pred)) + geom_line() +
-          geom_point(temp_data, mapping = aes(x = DOY, y = Width)) +
-          ylab("Width predicted") + theme_light()
-        ggsave(paste0("brnn_", i, ".png"))
+          geom_point(temp_data, mapping = aes(x = DOY, y = Width, alpha = note)) +
+          ylab("Width predicted") + theme_light() + guides(alpha = "none")
+
+        ggsave(paste0("brnn_", i, ".png"), width = 7, height = 6)
       }
 
     }
@@ -481,6 +534,9 @@ if (current_fitting_method == "gompertz"){
       gam_k <- temp_parameters$gam_k
       gam_sp <- temp_parameters$gam_sp
 
+      # specify the measurement type - so we can distinguish from added zeros
+      temp_data$note <- "raw measurement"
+
       # add zeros at the beggining
     if(add_zeros == TRUE){
 
@@ -508,22 +564,29 @@ if (current_fitting_method == "gompertz"){
       new_rows <- do.call(rbind, row_list)
       new_rows$DOY <- c(1:min_DOY)
       new_rows$Width <- 0
+      new_rows$note <- "added zero"
 
       temp_data <- rbind(new_rows, temp_data)
       temp_data$Width <- as.numeric(temp_data$Width)
     }
 
-      output <- gam(Width ~ s(DOY, k = gam_k, bs ="ps", sp = gam_sp),
+      output <- gam(Width ~ s(DOY, k = gam_k[1], bs ="ps", sp = gam_sp[1]),
                     data = temp_data, method = "REML")
 
       temp_data$Width_pred <- predict(output)
 
+      temp_data <- dplyr::arrange(temp_data, DOY)
+
       # plot(y = temp_data$Width, x = temp_data$DOY, main = i)
       # lines(y = temp_data$Width_pred, x = temp_data$DOY, type = "l")
 
       if (post_process == TRUE){
 
-      avg_fit <- mean(temp_data$Width_pred)
+      avg_fit <- ifelse(
+        add_zeros == TRUE, mean(temp_data$Width_pred),
+        mean(temp_data$Width_pred)/2 ) # That is a rule of thumb, but works nice for all the data
+
+
       max_fit <- max(temp_data$Width_pred)
 
       lagged_Width_pred <- temp_data$Width_pred[-length(temp_data$Width_pred)]
@@ -556,6 +619,7 @@ if (current_fitting_method == "gompertz"){
 
       for (J in 1:nrow(temp_data)){
 
+        J_shift <- ifelse(J == 1, 0, 1)
 
         if (temp_data[J, "Width_pred"] < 0.0001){
 
@@ -563,11 +627,11 @@ if (current_fitting_method == "gompertz"){
 
         } else if (temp_data[J, "first_diff"] < 0){
 
-          temp_data[J, "Width_pred"] <- temp_data[J - 1, "Width_pred"]
+          temp_data[J, "Width_pred"] <- temp_data[J - J_shift, "Width_pred"]
 
-        } else if ((temp_data[J, "Width_pred"] - temp_data[J - 1, "Width_pred"]) < 0){
+        } else if ((temp_data[J, "Width_pred"] - temp_data[J - J_shift, "Width_pred"]) < 0){
 
-          temp_data[J, "Width_pred"] <- temp_data[J - 1, "Width_pred"]
+          temp_data[J, "Width_pred"] <- temp_data[J - J_shift, "Width_pred"]
 
         } else {
 
@@ -577,8 +641,8 @@ if (current_fitting_method == "gompertz"){
 
         }
 
-     # plot(y = temp_data$Width, x = temp_data$DOY, main = i)
-     # lines(y = temp_data$Width_pred, x = temp_data$DOY, type = "l")
+       # plot(y = temp_data$Width, x = temp_data$DOY, main = i)
+       # lines(y = temp_data$Width_pred, x = temp_data$DOY, type = "l")
 
       }
 
@@ -588,10 +652,12 @@ if (current_fitting_method == "gompertz"){
       b_holder = b_holder + 1
 
       if (fitted_save == TRUE){
-        ggplot(temp_data, aes(x = DOY, y = Width_pred)) +
-          geom_line() + geom_point(temp_data, mapping = aes(x = DOY, y = Width)) +
-          ylab("Width predicted") + theme_light()
-        ggsave(paste0("gam_", i, ".png"))
+
+        ggplot(temp_data, aes(x = DOY, y = Width_pred)) + geom_line() +
+          geom_point(temp_data, mapping = aes(x = DOY, y = Width, alpha = note)) +
+          ylab("Width predicted") + theme_light() + guides(alpha = "none")
+
+        ggsave(paste0("gam_", i, ".png"), width = 7, height = 6)
       }
     }
 
@@ -607,7 +673,6 @@ if (current_fitting_method == "gompertz"){
   output_list <- list(fitted = do.call(rbind, list_temps),
                       gompertz_grid_search = final_parameters,
                       gompertz_grid_search_errors = errors_grid
-
                       )
 
   class(output_list) <- "xpsg"
